function [tout,yout] = ode4(F,t0,h,tfinal,y0)
% ODE4 Standard RK4 ODE solver of higher.
%   [tout,yout] = ODE4(F,t0,h,tfinal,y0) uses the standard Runge-Kutta 
%   method with a fixed step size h on the interval
%      t0 <= t <= tfinal
%   to solve
%      dy/dt = F(t,y)
%   with y(t0) = y0.
%
% Original (c) 2014 - 2015 The MathWorks, Inc.
% Modified 2022 UWB Pilsen

tout = t0 : h : tfinal;  % fixed time steps from t0 to tfinal
nt = length(tout);
yout = zeros(length(y0), nt);  % pre-allocation of output

y = y0;  % initial value
yout(1) = y; % the initial value is the first output as well

h2 = h/2;
h6 = h/6;  % will be used frequently
for k = 1:(nt-1)  % we fill positions 2...nt
    t = tout(k);  % time step t_k
    s1 = F(t,y);  % first estimate of function slope
    s2 = F(t+h2, y+h2*s1);  % second estimate of function slope
    s3 = F(t+h2, y+h2*s2);  % third estimate of function slope
    s4 = F(t+h, y+h*s3);  % fourth estimate of function slope
    y = y + h6*(s1 + 2*s2 + 2*s3 + s4);
    yout(k+1) = y;
end
